function [qgOmega,ifail]=calcOmega(firstTerm,secondTerm,thirdTerm,lon,lat,level,sigmaLoc,surfP,dens,w,f)
% compute the inversion of QG omega equation
%
% Inputs: firstTerm(lon,lat,level,time)          RHS of QG omega equation
%         secondTerm(lon,lat,level,time)
%         thirdTerm(lon,lat,level,time)
%         lon
%         lat
%         level
%         sigmaLoc(lon,lat,level,time)
%         surfP(lon,lat,time)                    surface pressure
%         dens(lon,lat,level,time)
%         w
%
% Outputs: qgOmega(lon,lat,level,time,4)
%          %The last dimension: 1-firstTerm, 2-secondTerm, 3-thridTerm, 4-topography 

%%
tmp=size(firstTerm);
nlon=tmp(1);
nlat=tmp(2);
nlev=tmp(3);
nt=tmp(4);

%% lower boundary for inversions of QGw equation
% lower boundary for inversions without topography
surfBC=zeros(nlon,nlat);  

%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
topoOmega=squeeze(w(:,:,nlev,:));
for iLev=nlev:-1:2
    tmp=squeeze(w(:,:,iLev-1,:));
    idx=find(surfP/100.0<=level(iLev));
    if isempty(idx)
        break;
    end
    topoOmega(idx)=tmp(idx);
end
%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    

%% inversion of QGw 
qgOmega=zeros(nlon,nlat,nlev,nt,4);
ifail=zeros(nlon,nlat,nt,4);
%qgW=zeros(size(qgOmega));

for iModel=1:4  
    switch iModel
        case 1
            RHS=firstTerm;                     % advection terms
        case 2
            RHS=secondTerm;      
        case 3
            RHS=thirdTerm;                     % adiabatic heating
        case 4
            RHS=zeros(nlon,nlat,nlev,nt);      % topography
    end
    
    
    for it=1:nt
        
        if iModel==4      % with topography
            [omegaTemp,myW,ifailTemp]=invertOmega(RHS(:,:,:,it),lon,lat,level,sigmaLoc(:,:,:,it),f,topoOmega(:,:,it),surfP(:,:,it),dens(:,:,:,it));
        else              % flat lower boundary
            [omegaTemp,myW,ifailTemp]=invertOmega(RHS(:,:,:,it),lon,lat,level,sigmaLoc(:,:,:,it),f,surfBC,surfP(:,:,it),dens(:,:,:,it));
        end
        qgOmega(:,:,:,it,iModel)=omegaTemp;
        ifail(:,:,it,iModel)=ifailTemp;
        if ifailTemp==1
            qgOmega(:,:,:,it,iModel)=0/0;  % if qgw dose not converge, set to NAN
            fprintf('iModel %1d: %3d day, ',iModel,it);
        end
	%qgW(:,:,:,it,iModel)=myW;
    end

end


