function [firstTerm,secondTerm,thirdTerm]=calcTerms(geopot,uGeostr,vGeostr,omega,sigmaLoc,lon,lat,level,time,f)
%
% DShaevitz 11/2014
% Compute three terms on the RHS of the QG omega equation
% [firstTerm,secondTerm,thirdTerm]=calcTerms(geopot,uGeostr,vGeostr,omega,sigmaLoc,lon,lat,level,time,f0)
%
% Inputs: geopot(lon,lat,level,time)   Geopotential
%         uGeostr(lon,lat,level,time)  u Geostrophic Velocity
%         vGeostr(lon,lat,level,time)  v Geostrophic Velocity 
%         omega(lon,lat,level,time)    Reanalysis omega
%         sigmaLoc(lon,lat,level,time) 4-Dim static stability
%         lon                          Longitude in degrees
%         lat                          Latitude in degrees
%         level                        Pressure in mb
%         time                         Time in matlab datenum
%         f                            f   
%
% Outputs: firstTerm                   Differential Vorticity Advection
%          secondTerm                  Laplacian of Temperature Advection
%          thirdTerm                   Laplacian of Diabatic Heating
%

firstTerm=calcFirstTerm(geopot,uGeostr,vGeostr,lon,lat,level,f);

secondTerm=calcSecondTerm(geopot,uGeostr,vGeostr,lon,lat,level);

thirdTerm=calcThirdTerm_sigmaLoc(geopot,uGeostr,vGeostr,omega,sigmaLoc,lon,lat,level,time);

