function [ RESULT , ifail ] = invlap3d(RHS,NX,NY,NZ,surfBC,surfP,level,AA,BB,CC,DD,EE,FF,GG)
% DShaevitz 11/2014 - Based on fortran code from ETH
% Invert 3D laplacian
% [ RESULT, ifail ] = invlap3d(RHS,NX,NY,NZ,surfBC,AA,BB,CC,DD,EE,FF,GG)

ITMAX = 50;
CONVERGED_CRITERION = 1.0e-2;
DELMAX  = 0.0;

ifail = 0;

SOLN=zeros(size(RHS));
R1=zeros(size(RHS));

%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
% for i=1:NX
%     for j=1:NY
%         for k=1:NZ
%             if level(k) > surfP(i,j)/100
%                 RHS(i,j,k)=0.;
%             end
%         end
%     end
% end
%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
level1=permute(repmat(level,[1,NX,NY]),[2,3,1]);
surfP1=repmat(surfP,[1,1,NZ]);
RHS(level1>surfP1/100)=0;
% id=level1>surfP1/100;
% AA(id)=0;BB(id)=0;CC(id)=0;DD(id)=0;EE(id)=0;FF(id)=0;GG(id)=0;
%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

% Initialise the starting approximation at every call
for IT = 1:ITMAX
   if (IT == 1)
      CONVERGED = false;
      SOLN(:)      = 0.0;
      R1(:)        = 0.0;      
      SOLN(:,:,end)=surfBC;
   end
   
   if (CONVERGED)
       return;
   end
   
   %^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
%    for K = 2:NZ-1
%       for J = 2:NY-1
%          for I = 2:NX-1               
%             if (DD(I,J,K) ~= 0.0)
%                R1(I,J,K) = RHS(I,J,K)- AA(I,J,K)*SOLN(I,J,K-1) - ...
%                     BB(I,J,K)*SOLN(I,J-1,K) - CC(I,J,K)*SOLN(I-1,J,K) - ...
%                     DD(I,J,K)*SOLN(I,J,K) - EE(I,J,K)*SOLN(I+1,J,K) - ...
%                     FF(I,J,K)*SOLN(I,J+1,K) - GG(I,J,K)*SOLN(I,J,K+1);
%             else
%                R1(I,J,K) = RHS(I,J,K) - SOLN(I,J,K);
%             end                        
%          end
%       end
%    end
   %^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
   SOLN_I0 = SOLN(1:NX-2, 2:NY-1, 2:NZ-1); %SOLN(I-1,J,K)
   SOLN_I2 = SOLN(3:NX-0, 2:NY-1, 2:NZ-1); %SOLN(I+1,J,K)
   SOLN_J0 = SOLN(2:NX-1, 1:NY-2, 2:NZ-1); %SOLN(I,J,K-1)
   SOLN_J2 = SOLN(2:NX-1, 3:NY-0, 2:NZ-1); %SOLN(I,J-1,K)   
   SOLN_K0 = SOLN(2:NX-1, 2:NY-1, 1:NZ-2); %SOLN(I,J+1,K)
   SOLN_K2 = SOLN(2:NX-1, 2:NY-1, 3:NZ-0); %SOLN(I,J,K+1)
   TMP_DD = DD(2:NX-1, 2:NY-1, 2:NZ-1);
   TMP_RHS = RHS(2:NX-1, 2:NY-1, 2:NZ-1);
   TMP_SOLN = SOLN(2:NX-1, 2:NY-1, 2:NZ-1);

   TMP_R1 = TMP_RHS- AA(2:NX-1,2:NY-1,2:NZ-1).*SOLN_K0- ...
       BB(2:NX-1,2:NY-1,2:NZ-1).*SOLN_J0 - CC(2:NX-1,2:NY-1,2:NZ-1).*SOLN_I0 - ...
       DD(2:NX-1,2:NY-1,2:NZ-1).*TMP_SOLN - EE(2:NX-1,2:NY-1,2:NZ-1).*SOLN_I2 - ...
       FF(2:NX-1,2:NY-1,2:NZ-1).*SOLN_J2 - GG(2:NX-1,2:NY-1,2:NZ-1).*SOLN_K2;
   id = find(TMP_DD == 0.0);
   TMP_R1(id) = TMP_RHS(id) - TMP_SOLN(id);
   R1(2:NX-1, 2:NY-1, 2:NZ-1) = TMP_R1;
   %^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^


   R1 = ellipticPDEsolver(IT,R1,NX,NY,NZ,AA,BB,CC,DD,EE,FF,GG);

   
% Updated solution
   DELMAX    = max(abs(R1(:))); 
   DELMAX2   = prctile(abs(R1(:)),99);
     
   SOLN = SOLN + R1;
   
   if ((IT > 1) && (DELMAX <= CONVERGED_CRITERION))
%        disp(['Convergence after ',num2str(IT),' iterations and DELMAX = ',num2str(DELMAX)]);
      RESULT    = SOLN;
      CONVERGED = true;
   end
   
   if DELMAX > 1.0e10
       disp('INVLAP3D (OMEGA EQN): Convergence not achieved');
       disp([' DELMAX = ',num2str(DELMAX)]);
       RESULT = SOLN;
       ifail  = 1;
       return
   end
      
end

% If we dont reach convergence, then we write the MISSING DATA VALUE
%  to the array
if (~CONVERGED)
    disp('INVLAP3D (OMEGA EQN): Convergence not achieved');
    disp([' DELMAX = ',num2str(DELMAX)]);
   RESULT = SOLN;
   ifail  = 1;
end

end

